import { buffer_tx, decode_tx } from '@scrow/tapscript/tx';
import { get_config } from '../config.js';
import { CoreWallet } from './wallet.js';
import { parse_args, run_cmd } from './cmd.js';
import { ensure_file_exists } from './util.js';
export class CoreClient {
    constructor(config = {}) {
        const opt = get_config(config);
        const { debug } = opt;
        if (opt.datapath === 'string' &&
            opt.cookiepath === undefined) {
            opt.cookiepath = `${opt.datapath}/${opt.network}/.cookie`;
        }
        this.params = [
            `-chain=${opt.network}`,
            ...opt.params,
            ...opt.cli_params
        ];
        if (opt.cookiepath !== undefined) {
            this.params.push(`-rpccookiefile=${opt.cookiepath}`);
        }
        if (opt.confpath !== undefined) {
            ensure_file_exists(opt.confpath);
            this.params.push(`-conf=${opt.confpath}`);
        }
        if (opt.rpcport !== undefined) {
            this.params.push(`-rpcport=${opt.rpcport}`);
        }
        this._opt = opt;
        if (debug)
            console.log('[debug] Initializing CLI with params:', this.params.join(' '));
    }
    get opt() {
        return this._opt;
    }
    get get_info() {
        return this.cmd('getblockchaininfo');
    }
    get list_wallets() {
        return this.cmd('listwallets');
    }
    get list_wallet_dir() {
        return this.cmd('listwalletdir');
    }
    async cmd(method, args = [], params = []) {
        const { debug } = this.opt;
        const p = [...this.params, ...params];
        p.push(...parse_args(method, args));
        if (debug) {
            const offset = this.params.length;
            console.log('[debug] cmd:', p.slice(offset).join(' '));
        }
        return run_cmd(this.opt.clipath, p);
    }
    async scan_txout(action, ...desc) {
        return this.cmd('scantxoutset', [action, JSON.stringify(desc)]);
    }
    async scan_addr(addr) {
        return this.scan_txout('start', `addr(${addr})`).then(res => {
            const { success, unspents } = res;
            return (success) ? unspents : [];
        });
    }
    async mine_blocks(count = 1, addr) {
        if (this.opt.network !== 'regtest') {
            throw new Error('You can only generate funds on regtest network!');
        }
        if (addr === undefined) {
            const wallet = await this.get_wallet('faucet');
            const { address } = await wallet.get_address('faucet');
            addr = address;
        }
        return this.cmd('generatetoaddress', [count, addr]);
    }
    async get_tx(txid) {
        const { hex, ...meta } = await this.cmd('getrawtransaction', [txid, true]);
        return { hex, meta, txdata: decode_tx(hex) };
    }
    async get_wallet(name) {
        if (!await this.is_wallet_loaded(name)) {
            if (!await this.is_wallet_created(name)) {
                await this.create_wallet(name);
            }
            else {
                await this.load_wallet(name);
            }
        }
        return new CoreWallet(this, name);
    }
    async is_wallet_loaded(name) {
        return this.list_wallets
            .then((wallets) => Array.isArray(wallets) && wallets.includes(name));
    }
    async is_wallet_created(name) {
        return this.list_wallet_dir
            .then((res) => res.wallets.find(e => e.name === name));
    }
    async load_wallet(name) {
        const res = await this.cmd('loadwallet', name);
        if (res.warning !== undefined || res.name !== name) {
            throw new Error(`Wallet failed to load cleanly: ${JSON.stringify(res, null, 2)}`);
        }
    }
    async create_wallet(name, config = {}) {
        const payload = { wallet_name: name, ...config };
        const res = await this.cmd('createwallet', payload);
        if ((res.warning !== undefined && res.warning !== '') ||
            res.name !== name) {
            throw new Error(`Wallet failed to create: ${JSON.stringify(res, null, 2)}`);
        }
    }
    async publish_tx(txdata) {
        const txhex = buffer_tx(txdata).hex;
        return this.cmd('sendrawtransaction', [txhex]);
    }
}
//# sourceMappingURL=client.js.map