import { derive_key } from '@cmdcode/crypto-tools/hd';
import { segwit, taproot } from '@scrow/tapscript/sighash';
import { create_tx, create_vin } from '@scrow/tapscript/tx';
import assert from 'assert';
import { parse_desc, parse_descriptor } from './descriptors.js';
const SAT_MULTI = 100_000_000;
const DEFAULT_SORTER = () => Math.random() > 0.5 ? 1 : -1;
export class CoreWallet {
    constructor(client, wallet_name = 'test_wallet') {
        this._client = client;
        this._name = wallet_name;
        this._init = false;
    }
    get client() {
        return this._client;
    }
    get name() {
        return this._name;
    }
    get balance() {
        return this.cmd('getbalance')
            .then(bal => Number(bal) * SAT_MULTI);
    }
    get network() {
        return this.client.opt.network;
    }
    get newaddress() {
        return new Promise(async (res) => {
            const addr = await this.cmd('getnewaddress');
            const data = await this.parse_address(addr);
            res(data);
        });
    }
    get utxos() {
        return this.cmd('listunspent')
            .then(e => e.map(x => { return { ...x, sats: Math.round(x.amount * SAT_MULTI) }; }));
    }
    get xprvs() {
        return this.cmd('listdescriptors', 'true')
            .then(({ descriptors }) => descriptors.map(x => parse_descriptor(x)));
    }
    get xpubs() {
        return this.cmd('listdescriptors')
            .then(({ descriptors }) => descriptors.map(x => parse_descriptor(x)));
    }
    async cmd(method, args = [], params = []) {
        const p = [`-rpcwallet=${this.name}`, ...params];
        return this.client.cmd(method, args, p);
    }
    async gen_address(config = {}) {
        return this.cmd('getnewaddress', config);
    }
    async get_address(label) {
        return new Promise(async (res) => {
            let address;
            try {
                const res = await this.cmd('getaddressesbylabel', [label]);
                const addrs = Object.keys(res);
                address = (addrs.length !== 0)
                    ? addrs[0]
                    : await this.gen_address({ label });
            }
            catch (err) {
                address = await this.gen_address({ label });
            }
            assert.ok(typeof address === 'string');
            res(this.parse_address(address));
        });
    }
    async parse_address(address) {
        return this.cmd('getaddressinfo', [address]);
    }
    async send_funds(address, amt) {
        const amount = amt / SAT_MULTI;
        const config = { address, amount, estimate_mode: 'economical' };
        return this.cmd('sendtoaddress', config);
    }
    async ensure_funds(min_bal, blocks = 110) {
        if (this.network === 'regtest') {
            const bal = await this.balance;
            if (bal <= min_bal) {
                await this.generate_funds(blocks);
                return this.ensure_funds(min_bal, blocks);
            }
        }
    }
    async generate_funds(blocks, address) {
        if (address === undefined) {
            address = (await this.newaddress).address;
        }
        return this._client.cmd('generatetoaddress', [blocks ?? 110, address]);
    }
    async get_xprv(label) {
        const xprvs = await this.xprvs;
        return xprvs.find(e => label === e.label);
    }
    async get_signer(desc) {
        const { parent_label, fullpath, keytype } = parse_desc(desc);
        const xprv = await this.get_xprv(parent_label);
        assert.ok(xprv?.extkey?.seckey !== undefined);
        const { seckey: master_key, code } = xprv.extkey;
        const { seckey, pubkey } = derive_key(fullpath, master_key, code, true);
        assert.ok(seckey !== null);
        const signer = (keytype === 'tr') ? taproot : segwit;
        const sign_tx = (txdata, config) => {
            return signer.sign_tx(seckey, txdata, config);
        };
        return { pubkey, sign_tx };
    }
    async select_utxos(amount, coinsorter = DEFAULT_SORTER) {
        const selected = [];
        let total = 0;
        const utxos = await this.utxos;
        utxos.sort(coinsorter);
        for (const utxo of utxos) {
            selected.push(utxo);
            total += utxo.sats;
            if (total >= amount)
                return selected;
        }
        throw new Error('Insufficient funds!');
    }
    async fund_tx(templ, config = {}, txfee = 1000) {
        const txdata = create_tx(templ);
        const vamt = txdata.vout.reduce((prev, curr) => Number(curr.value) + prev, 0);
        const utxos = await this.select_utxos(vamt + txfee);
        const total = utxos.reduce((prev, curr) => curr.sats + prev, 0);
        const change = total - vamt - txfee;
        const change_addr = await this.newaddress;
        const changePubKey = change_addr.scriptPubKey;
        txdata.vout.push({
            value: BigInt(change),
            scriptPubKey: changePubKey
        });
        for (let i = 0; i < utxos.length; i++) {
            const { desc, txid, vout, sats, scriptPubKey } = utxos[i];
            const { pubkey, sign_tx } = await this.get_signer(desc);
            const prevout = { value: sats, scriptPubKey };
            const txinput = create_vin({ txid, vout, prevout });
            const txconfig = { sigflag: 0x81, pubkey, txinput };
            const signature = sign_tx(txdata, { ...txconfig, ...config });
            const witness = [signature.hex];
            if (desc.startsWith('wpkh'))
                witness.push(pubkey.hex);
            txdata.vin.push({ ...txinput, witness });
        }
        return txdata;
    }
}
//# sourceMappingURL=wallet.js.map